@@ -25,7 +25,9 @@ import importlib.resources as pkg_resources
 import json
 import logging
 import os
+import random
 import re
+import string
 import sys
 import textwrap
 import traceback
@@ -42,7 +44,7 @@ from owlready2 import destroy_entity, get_ontology, onto_path, types,\
                       FunctionalProperty, InverseFunctionalProperty,\
                       TransitiveProperty, SymmetricProperty, AsymmetricProperty,\
                       ReflexiveProperty, IrreflexiveProperty, ThingClass,\
-                      Not, Inverse, base, locstr
+                      Not, Inverse, base, locstr, And, Or, class_construct
 from pyvis.network import Network
 
 from . import config
@@ -53,7 +55,7 @@ LOGFILE = datetime.datetime.now().strftime("%Y-%m-%d_%H-%M-%S")+"_ontor.log"
 logging.basicConfig(filename=LOGFILE, level=logging.DEBUG)
 
 
-def load_csv(csv_file: str, load_first_line: bool=False) -> list:
+def load_csv(csv_file: str, load_first_line: bool = False) -> list:
     """ load data from CSV file
 
     :param csv_file: input CSV file
@@ -68,7 +70,7 @@ def load_csv(csv_file: str, load_first_line: bool=False) -> list:
     return data
 
 
-def load_json(json_file: str) -> dict:
+def load_json(json_file: str) -> typing.Union[dict, list]:
     """ load data from JSON file
 
     :param json_file: input JSON file
@@ -94,12 +96,21 @@ def cleanup(complete: bool, *extensions: str) -> None:
             os.remove(os.path.join(this_dir, f))
 
 
+class InfoException(Exception):
+    """ exception for invalid ontor inputs
+    """
+
+    def __init__(self, **kwargs: str):
+        Exception.__init__(self)
+        self.info = kwargs
+
+
 class OntoEditor:
     """ create, load, and edit ontologies
     """
 
     # NOTE: _prop_types corresponds to owlready2.prop._TYPE_PROPS; defined here to ensure order
-    _prop_types = [FunctionalProperty, InverseFunctionalProperty, TransitiveProperty,\
+    _prop_types = [FunctionalProperty, InverseFunctionalProperty, TransitiveProperty,
                    SymmetricProperty, AsymmetricProperty, ReflexiveProperty, IrreflexiveProperty]
     _dp_range_types = {"boolean": bool,
                        "float": float,
@@ -109,7 +120,7 @@ class OntoEditor:
                        "time": datetime.time,
                        "datetime": datetime.datetime}
 
-    def __init__(self, iri: str, path: str, import_paths: list=None) -> None:
+    def __init__(self, iri: str, path: str, import_paths: list = None) -> None:
         """ tries to load onto from file specified, creates new file if none is available
 
         :param iri: ontology's IRI
@@ -121,16 +132,16 @@ class OntoEditor:
         self.filename = path.split(sep="/")[-1]
         self.logger = logging.getLogger(self.filename.split(".")[0])
         self.query_prefixes = pkg_resources.read_text(queries, 'prefixes.sparql')
-        onto_path.extend(list(set([path.rsplit("/", 1)[0]]) - set(onto_path)))
+        onto_path.extend(list({path.rsplit("/", 1)[0]} - set(onto_path)))
         if import_paths:
             onto_path.extend(list(set(import_paths) - set(onto_path)))
         self.onto_world = World()
         try:
             self.onto = self.onto_world.get_ontology(self.path).load()
             self.logger.info("successfully loaded ontology specified")
-        except:
+        except FileNotFoundError:
             self.onto = self.onto_world.get_ontology(self.iri)
-            self.onto.save(file = self.path)
+            self.onto.save(file=self.path)
             self.logger.info("ontology file did not exist - created a new one")
 
     @contextmanager
@@ -158,10 +169,11 @@ class OntoEditor:
 
     def _reload_from_file(self) -> None:
         try:
-            self.onto = get_ontology(self.path).load()
+            self.onto_world = World()
+            self.onto = self.onto_world.get_ontology(self.path).load()
             self.logger.info("successfully reloaded ontology from file")
-        except:
-            self.logger.info("ontology file did not exist")
+        except FileNotFoundError:
+            self.logger.error("ontology file did not exist")
             sys.exit(1)
 
     def add_import(self, other_path: str) -> None:
@@ -191,10 +203,10 @@ class OntoEditor:
         """ saves with same filename, but as ntriples
         """
         ntpath = self.path.rsplit(".", 1)[0] + ".nt"
-        self.onto.save(file = ntpath, format = "ntriples")
+        self.onto.save(file=ntpath, format="ntriples")
 
     def get_elems(self) -> list:
-        """ get classes, object properties, datatype properties, and intances
+        """ get classes, object properties, datatype properties, and instances
 
         :return: nodes and edges from onto
         """
@@ -238,6 +250,30 @@ class OntoEditor:
             axioms.append(self.query_onto(self._build_query(query_ax)))
         return axioms
 
+    def _create_notion(self, name, parent, elem_type) -> type:
+        """ load a notion from the ontology or create a new one if not yet available
+        works for classes, object properties, and data properties
+
+        :param name: name of the notion
+        :param parent: name of the parent notion
+        :param elem_type: indicates class, op, or dp
+        :return: notion
+        """
+        type_dict = {
+            "c": Thing,
+            "o": ObjectProperty,
+            "d": DataProperty,
+        }
+        with self.onto:
+            if name and not parent:
+                notion = types.new_class(name, (type_dict[elem_type], ))
+            elif name and parent:
+                notion = types.new_class(name, (self.onto[parent], ))
+            else:
+                self.logger.warning(f"unexpected info: {name, parent, elem_type}")
+                raise InfoException
+        return notion
+
     def add_taxo(self, class_tuples: list) -> None:
         """ add taxonomy to onto
 
@@ -245,13 +281,12 @@ class OntoEditor:
         """
         with self.onto:
             for clst in class_tuples:
-                if clst[0] and not clst[1]:
-                    my_class = types.new_class(clst[0], (Thing, ))
-                elif clst[0] and clst[1]:
-                    my_class = types.new_class(clst[0], (self.onto[clst[1]], ))
-                else:
-                    self.logger.warning(f"no class defined: {clst}")
-        self.onto.save(file = self.path)
+                try:
+                    my_class = self._create_notion(clst[0], clst[1], "c")
+                except (TypeError, InfoException):
+                    self.logger.warning(f"unexpected class info: {clst}")
+                    continue
+        self.onto.save(file=self.path)
 
     @staticmethod
     def class_dict_to_tuple_list(cls_dict: dict) -> list:
@@ -264,40 +299,89 @@ class OntoEditor:
         """
         return [[subcls, supercls] for supercls in cls_dict.keys() for subcls in cls_dict[supercls]]
 
-    def add_axioms(self, axiom_tuples: list) -> None:
+    def _combine_axioms(self, axs: dict) -> tuple:
+        """ define complex axioms, i.e., elementary axioms that are logically combined
+
+        :param axs: input for axioms, either simple list or of the form {"or": [ax1, "and": [ax2, ax3]]}
+        :return: combined restriction, restriction type (equivalence or subclass), and class
+        """
+        res, comb, cls = [], None, None
+        res_type_set, res_type = False, None
+        res_type_vals = {
+            None: False,
+            False: False,
+            True: True,
+        }
+        assert len(axs.keys()) == 1, f"more than one operator defined for axiom: {axs}"
+        operator = list(axs.keys())[0]
+        assert operator in ["and", "or"], f"invalid key for axiom combination: {operator}"
+        for axiom in axs[operator]:
+            if isinstance(axiom, list):
+                if not res_type_set:
+                    res_type_set, res_type = True, res_type_vals[axiom[-1]]
+                else:
+                    assert res_type_vals[axiom[-1]] == res_type,\
+                        f"restriction types (subsumption vs equivalence) do not match: {axs}"
+                if not cls:
+                    cls = axiom[0]
+                else:
+                    assert axiom[0] == cls, f"aggregated restriction does not always refer to same class: {axs}"
+                res.append(self._tuple_to_res(axiom[1], [self.onto[axiom[2]], axiom[3], axiom[4], axiom[5], axiom[13]],
+                                              [self.onto[axiom[6]]], axiom[7:13], axiom))
+            elif isinstance(axiom, dict):
+                res.append(self._combine_axioms(axiom)[0])
+        if operator == "and":
+            comb = And(res)
+        elif operator == "or":
+            comb = Or(res)
+        return comb, res_type, cls
+
+    def add_axioms(self, axioms: list) -> None:
         """ add entire axioms to onto
         NOTE: only one axiom may be specified at once
         NOTE: no error handling implemented for input tuples
-        NOTE: complex axioms, i.e., intersections and unions, are currently not supported
 
-        :param axiom_tuples: list of tuples of the form [class, superclass, property,
+        :param axioms: list of tuples of the form [class, superclass, property,
             inverted(bool), cardinality type, cardinality, op-object, dp-range,
             dp-min-ex, dp-min-in, dp-exact, dp-max-in, dp-max-ex, negated(bool),
             equivalence(bool)]
+            may also include dicts containing aggregate axioms of the form {"or": [ax1, "and": [ax2, ax3]]}
         """
         with self.onto:
-            for axiom in axiom_tuples:
-                my_class = self.onto[axiom[0]]
-                if axiom[0] and axiom[1] and axiom[-1]:
-                    my_class.equivalent_to.append(self.onto[axiom[1]])
-                if not axiom[2] and not axiom[4] and not axiom[5] and not axiom[5] == 0 and not axiom[6]:
-                    continue
-                if all(axiom[i] for i in [2,4,6]) or all(axiom[i] for i in [2,4,7]):
-                    if axiom[-1]:
+            for axiom in axioms:
+                if isinstance(axiom, list):
+                    my_class = self.onto[axiom[0]]
+                    if not any(axiom[i] for i in [1, 2, 4, 5, 6]) and not axiom[5] == 0:
+                        continue
+                    if all(axiom[i] for i in [0, 1, -1]) or all(axiom[i] for i in [2, 4, 6])\
+                            or all(axiom[i] for i in [2, 4, 7]):
+                        if axiom[-1]:
+                            current_axioms = my_class.equivalent_to
+                        else:
+                            current_axioms = my_class.is_a
+                        res = self._tuple_to_res(axiom[1], [self.onto[axiom[2]], axiom[3], axiom[4], axiom[5], axiom[13]],
+                                                 [self.onto[axiom[6]]], axiom[7:13], axiom)
+                        if res:
+                            current_axioms.append(res)
+                    else:
+                        self.logger.warning(f"unexpected input: {axiom}")
+                elif isinstance(axiom, dict):
+                    comb, res_type, cls = self._combine_axioms(axiom)
+                    my_class = self.onto[cls]
+                    if res_type:
                         current_axioms = my_class.equivalent_to
                     else:
                         current_axioms = my_class.is_a
-                    self._add_restr_to_def(current_axioms, [self.onto[axiom[2]],\
-                                           axiom[3], axiom[4], axiom[5], axiom[13]],\
-                                           [self.onto[axiom[6]]], axiom[7:13], axiom)
-                else:
-                    self.logger.warning(f"unexpected input: {axiom}")
-        self.onto.save(file = self.path)
+                    if comb:
+                        current_axioms.append(comb)
+                    else:
+                        self.logger.warning(f"unexpected input: {axiom}")
+        self.onto.save(file=self.path)
 
-    def _add_restr_to_def(self, current_axioms: list, resinfo: list, opinfo: list,
-                          dpinfo: list, axiom: list) -> None:
+    def _tuple_to_res(self, supercls: str, resinfo: list, opinfo: list, dpinfo: list, axiom: list)\
+            -> typing.Union[type(class_construct), None]:
         """
-        :param current_axioms: list of an element's current axioms - equivalent_to or is_a
+        :param supercls: parent class or equivalent class, depending on equiv parameter
         :param resinfo: list with general restriction info [prop, inverted, p_type,
             cardin, negated]
         :param opinfo: list with op restriction info [op-object]
@@ -305,28 +389,30 @@ class OntoEditor:
             exact, maxin, maxex]
         :param axiom: list with complete axiom info
         """
-        if any(opinfo) and not any(dpinfo):
+        if supercls and not any(opinfo) and not any(dpinfo):
+            return self.onto[supercls]
+        elif any(opinfo) and not any(dpinfo):
             obj = opinfo[0]
         elif not any(opinfo) and any(dpinfo):
             obj = None
             if resinfo[1]:
                 self.logger.warning(f"invalid dp constraint - dp may not be inverted: {axiom}")
-                return
+                return None
             if resinfo[2] in ["some", "only"]:
                 obj = self._dp_constraint(dpinfo)
             elif resinfo[2] in ["value"] and dpinfo[3]:
                 obj = self._dp_range_types[dpinfo[0]](dpinfo[3])
             if obj is None:
                 self.logger.warning(f"invalid dp constraint: {axiom}")
-                return
+                return None
             if resinfo[2] in ["exactly", "max", "min"]:
                 # NOTE: this may be resolved in future versions of Owlready2
                 self.logger.warning("qualified cardinality restrictions currently not "
                                     f"supported for DPs: {axiom}")
-                return
+                return None
         else:
             self.logger.warning(f"restriction includes both op and dp: {axiom}")
-            return
+            return None
         if resinfo[1]:
             resinfo[0] = Inverse(resinfo[0])
         if resinfo[2] in ["some", "only", "value"] and not resinfo[3] and not resinfo[3] == 0:
@@ -335,10 +421,10 @@ class OntoEditor:
             res = getattr(resinfo[0], resinfo[2])(resinfo[3], obj)
         else:
             self.logger.warning(f"unexpected cardinality definition: {axiom}")
-            return
+            return None
         if resinfo[4]:
             res = Not(res)
-        current_axioms.append(res)
+        return res
 
     def _dp_constraint(self, dpres: list) -> typing.Optional[ConstrainedDatatype]:
         """
@@ -351,37 +437,37 @@ class OntoEditor:
             self.logger.warning(f"unexpected dp range: {dpres}")
         if self._check_available_vals(dpres, [0]):
             dp_range = self._dp_range_types[dpres[0]]
-        elif self._check_available_vals(dpres, [0,3]):
-            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],\
-                                           min_inclusive=dpres[3],\
+        elif self._check_available_vals(dpres, [0, 3]):
+            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],
+                                           min_inclusive=dpres[3],
                                            max_inclusive=dpres[3])
-        elif self._check_available_vals(dpres, [0,1,4]):
-            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],\
-                                           min_exclusive=dpres[1],\
+        elif self._check_available_vals(dpres, [0, 1, 4]):
+            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],
+                                           min_exclusive=dpres[1],
                                            max_inclusive=dpres[4])
-        elif self._check_available_vals(dpres, [0,1,5]):
-            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],\
-                                           min_exclusive=dpres[1],\
+        elif self._check_available_vals(dpres, [0, 1, 5]):
+            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],
+                                           min_exclusive=dpres[1],
                                            max_exclusive=dpres[5])
-        elif self._check_available_vals(dpres, [0,2,4]):
-            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],\
-                                           min_inclusive=dpres[2],\
+        elif self._check_available_vals(dpres, [0, 2, 4]):
+            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],
+                                           min_inclusive=dpres[2],
                                            max_inclusive=dpres[4])
-        elif self._check_available_vals(dpres, [0,2,5]):
-            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],\
-                                           min_inclusive=dpres[2],\
+        elif self._check_available_vals(dpres, [0, 2, 5]):
+            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],
+                                           min_inclusive=dpres[2],
                                            max_exclusive=dpres[5])
-        elif self._check_available_vals(dpres, [0,1]):
-            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],\
+        elif self._check_available_vals(dpres, [0, 1]):
+            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],
                                            min_exclusive=dpres[1])
-        elif self._check_available_vals(dpres, [0,2]):
-            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],\
+        elif self._check_available_vals(dpres, [0, 2]):
+            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],
                                            min_inclusive=dpres[2])
-        elif self._check_available_vals(dpres, [0,4]):
-            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],\
+        elif self._check_available_vals(dpres, [0, 4]):
+            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],
                                            max_inclusive=dpres[4])
-        elif self._check_available_vals(dpres, [0,5]):
-            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],\
+        elif self._check_available_vals(dpres, [0, 5]):
+            dp_range = ConstrainedDatatype(self._dp_range_types[dpres[0]],
                                            max_exclusive=dpres[5])
         else:
             self.logger.warning(f"unexpected dp range restriction: {dpres}")
@@ -397,9 +483,32 @@ class OntoEditor:
         indices = [x for x, _ in enumerate(values)]
         assert all(x in indices for x in expected_values), "invalid expected_values"
         test = all(values[i] for i in expected_values) and\
-               not any(values[i] for i in [e for e in indices if not e in expected_values])
+               not any(values[i] for i in [e for e in indices if e not in expected_values])
         return test
 
+    def add_gcas(self, gcas: list) -> None:
+        """ workaround for representing General Class Axioms
+        adds two helper classes, each defined via an axiom, that are defined to be equivalent
+        helper classes are denoted with an underscore
+
+        :param gcas: list of two-tuples with axioms as defined by add_axioms()
+        """
+        with self.onto:
+            class GcaHelper(Thing):
+                comment = ["Helper class for workaround to represent General Class Axioms"]
+            for gca in gcas:
+                for a in gca:
+                    gh_name = "_" + ''.join(random.choices(string.ascii_letters + string.digits, k=16))
+                    a.insert(0, gh_name)
+                    a.insert(1, "GcaHelper")
+                    assert a[-1] is True, "GCAs must be equivalented with auxiliary classes for inferences to work"
+                self.add_taxo([a[:2] for a in gca])
+                for a in gca:
+                    a[1] = None
+                gca.append([gca[0][0], gca[1][0]] + [None]*12 + [True])
+                self.add_axioms(gca)
+        self.onto.save(file=self.path)
+
     def add_ops(self, op_tuples: list) -> None:
         """ add object properties including their axioms to onto
         NOTE: only one inverse_prop can be processed per tuple
@@ -410,12 +519,11 @@ class OntoEditor:
         """
         with self.onto:
             for op in op_tuples:
-                if op[0] and not op[1]:
-                    my_op = types.new_class(op[0], (ObjectProperty, ))
-                elif op[0] and op[1]:
-                    my_op = types.new_class(op[0], (self.onto[op[1]], ))
-                else:
+                try:
+                    my_op = self._create_notion(op[0], op[1], "o")
+                except (TypeError, InfoException):
                     self.logger.warning(f"unexpected op info: {op}")
+                    continue
                 if op[2]:
                     my_op.domain.append(self.onto[op[2]])
                 if op[3]:
@@ -425,7 +533,7 @@ class OntoEditor:
                         my_op.is_a.append(self._prop_types[count])
                 if op[-1]:
                     my_op.inverse_property = self.onto[op[11]]
-        self.onto.save(file = self.path)
+        self.onto.save(file=self.path)
 
     def add_dps(self, dp_tuples: list) -> None:
         """ add datatype properties including their axioms to onto
@@ -436,11 +544,8 @@ class OntoEditor:
         with self.onto:
             for dp in dp_tuples:
                 try:
-                    if dp[0] and not dp[1]:
-                        my_dp = types.new_class(dp[0], (DataProperty, ))
-                    elif dp[0] and dp[1]:
-                        my_dp = types.new_class(dp[0], (self.onto[dp[1]], ))
-                except:
+                    my_dp = self._create_notion(dp[0], dp[1], "d")
+                except (TypeError, InfoException):
                     self.logger.warning(f"unexpected dp info: {dp}")
                     continue
                 if dp[2]:
@@ -448,7 +553,7 @@ class OntoEditor:
                 if dp[3]:
                     try:
                         my_dp.domain.append(self.onto[dp[3]])
-                    except:
+                    except Exception:
                         self.logger.warning(f"unexpected dp domain: {dp}")
                 if any(dp[4:]):
                     dprange = self._dp_constraint(dp[4:])
@@ -457,7 +562,7 @@ class OntoEditor:
                     else:
                         self.logger.warning(f"unexpected dp range: {dp}")
                         continue
-        self.onto.save(file = self.path)
+        self.onto.save(file=self.path)
 
     def add_instances(self, instance_tuples: list) -> None:
         """ add instances and their relations to onto
@@ -488,7 +593,7 @@ class OntoEditor:
                     self._add_instance_relation(my_instance, pred, val)
                 else:
                     self.logger.warning(f"unexpected triple: {inst}")
-        self.onto.save(file = self.path)
+        self.onto.save(file=self.path)
 
     @staticmethod
     def _add_instance_relation(subj, pred, obj) -> None:
@@ -510,9 +615,9 @@ class OntoEditor:
                 try:
                     func = funcs[ds[0]]
                     func([self.onto[elem] for elem in ds[1]])
-                except:
+                except KeyError:
                     self.logger.warning(f"unknown distinction type {ds[0]}")
-        self.onto.save(file = self.path)
+        self.onto.save(file=self.path)
 
     def remove_elements(self, elem_list: list) -> None:
         """ remove elements, all their descendents and (in case of classes) instances,
@@ -529,9 +634,9 @@ class OntoEditor:
                     if desc != self.onto[elem]:
                         destroy_entity(desc)
                 destroy_entity(self.onto[elem])
-        self.onto.save(file = self.path)
+        self.onto.save(file=self.path)
 
-    def add_label(self, name: str, label: str, lang: str=None) -> None:
+    def add_label(self, name: str, label: str, lang: str = None) -> None:
         """ add label in language specified as localized string, defaults to
         regular string if no language is specified
 
@@ -545,7 +650,7 @@ class OntoEditor:
         desc = entity.label
         self._add_description_generic(desc, label, lang)
 
-    def add_annotation(self, name: str, comment: str, lang: str=None) -> None:
+    def add_annotation(self, name: str, comment: str, lang: str = None) -> None:
         """ add annotation in language specified as localized string, defaults to
         regular string if no language is specified
 
@@ -580,9 +685,9 @@ class OntoEditor:
             desc_list.append(locstr(description, lang=lang))
         else:
             desc_list.append(description)
-        self.onto.save(file = self.path)
+        self.onto.save(file=self.path)
 
-    def remove_from_taxo(self, elem_list: list, reassign: bool=True) -> None:
+    def remove_from_taxo(self, elem_list: list, reassign: bool = True) -> None:
         """ remove a class from the taxonomy, but keep all subclasses and instances
         by relating them to parent
         NOTE: elem is not replaced in axioms bc this may be semantically incorrect
@@ -609,9 +714,9 @@ class OntoEditor:
                 for ind in individuals:
                     ind.is_a.append(parent[0])
                 destroy_entity(self.onto[elem])
-        self.onto.save(file = self.path)
+        self.onto.save(file=self.path)
 
-    def get_class_restrictions(self, class_name: str, res_type: str="is_a", res_only: bool=True) -> list:
+    def get_class_restrictions(self, class_name: str, res_type: str = "is_a", res_only: bool = True) -> list:
         """ retrieve restrictions on specific class by restriction type
 
         :param class_name: name of the class for which restrictions shall be returned
@@ -665,7 +770,7 @@ class OntoEditor:
             if not prop or prop and r.property == prop:
                 cls_restrictions.remove(r)
 
-    def reasoning(self, reasoner: str="hermit", save: bool=False, debug: bool=False) -> list:
+    def reasoning(self, reasoner: str = "hermit", save: bool = False, debug: bool = False) -> list:
         """ run reasoner to check consistency and infer new facts
 
         :param reasoner: reasoner can be eiter hermit or pellet
@@ -686,7 +791,7 @@ class OntoEditor:
                         sync_reasoner_hermit([inf_onto])
                     elif reasoner == "pellet":
                         # pellet explanations are generated if debug is set to >=2
-                        sync_reasoner_pellet([inf_onto], infer_property_values=True,\
+                        sync_reasoner_pellet([inf_onto], infer_property_values=True,
                                              infer_data_property_values=True, debug=debug+1)
                 inconsistent_classes = list(inf_onto.inconsistent_classes())
             except Exception as exc:
@@ -763,7 +868,7 @@ class OntoEditor:
                 debug_onto = debug.get_ontology(self.path).load()
                 with debug_onto:
                     try:
-                        sync_reasoner_pellet([debug_onto], infer_property_values=True,\
+                        sync_reasoner_pellet([debug_onto], infer_property_values=True,
                                              infer_data_property_values=True, debug=2)
                     except base.OwlReadyInconsistentOntologyError as err:
                         self.logger.error(repr(err))
@@ -780,7 +885,7 @@ class OntoEditor:
             ax_msg = "Potentially inconsistent axiom: "
             for rel in "is_a", "equivalent_to":
                 self._interactively_delete_axs_by_rel(rel, inconsistent_classes, pot_probl_ax, ax_msg)
-            self.onto.save(file = self.path)
+            self.onto.save(file=self.path)
             self.debug_onto(reasoner, assume_correct_taxo)
 
     def _get_incon_class_res(self, restype: str, inconsistent_classes: list) -> list:
@@ -808,7 +913,7 @@ class OntoEditor:
                     # IDEA: instead of simply deleting axioms, also allow user to edit them
 
     @staticmethod
-    def _bool_user_interaction(question: str, info: str=None) -> bool:
+    def _bool_user_interaction(question: str, info: str = None) -> bool:
         """ simple CLI for yes/ no/ quit interaction
         """
         answer = {"y": True,
@@ -841,7 +946,7 @@ class OntoEditor:
         :param coloring: dict with colors as keys and lists of nodes as values
         :return: nxgraph for the ontology including labels and coloring
         """
-        nxgraph = nx.from_pandas_edgelist(df, source="subject", target="object",\
+        nxgraph = nx.from_pandas_edgelist(df, source="subject", target="object",
                                           edge_attr="predicate", create_using=nx.MultiDiGraph())
         # manually set predicates as labels
         for e in nxgraph.edges.items():
@@ -870,8 +975,8 @@ class OntoEditor:
         df = pd.DataFrame(clean_data, columns=['subject', 'predicate', 'object'])
         return df
 
-    def _plot_nxgraph(self, nxgraph: nx.MultiDiGraph, open_html: bool=False,
-                      interactive: bool=False) -> None:
+    def _plot_nxgraph(self, nxgraph: nx.MultiDiGraph, open_html: bool = False,
+                      interactive: bool = False) -> None:
         """ create html file for the network's plot
 
         :param nxgraph: networkx graph including the ontology's triples
@@ -889,9 +994,9 @@ class OntoEditor:
         else:
             net.write_html(html_name)
 
-    def _config_plot_query_body(self, classes: list=None, properties: list=None,
-                                focusnode: str=None, radius: int=None, include_class_res: bool=True,
-                                show_class_descendants: bool=True) -> str:
+    def _config_plot_query_body(self, classes: list = None, properties: list = None,
+                                focusnode: str = None, radius: int = None, include_class_res: bool = True,
+                                show_class_descendants: bool = True) -> str:
         """ configure body for SPARQL query that identifies triples for plot
 
         :param classes: classes to be returned including their instances
@@ -914,15 +1019,18 @@ class OntoEditor:
 
         def _sparql_set_values(node, values):
             return "VALUES ?" + node + " {rdf:type rdfs:subClassOf " + " ".join([":" + v for v in values]) + "} . "
+
         def _sparql_set_in(node, values, sep=None):
             if not sep:
                 sep = ""
             return "FILTER ( ?" + node + " IN (" + ", ".join([sep + v for v in values]) + ") ) . "
+
         querypt_class_rels = ("?s rdfs:subClassOf | owl:equivalentClass ?res . \n"
                               "?res a owl:Restriction . \n"
                               "?res owl:onProperty ?p . \n"
                               "?res owl:onClass | owl:someValuesFrom | owl:allValuesFrom | owl:hasValue ?o . ")
         querypt1 = "SELECT DISTINCT ?s ?p ?o WHERE {\n"
+
         if include_class_res:
             # NOTE: only atomic axioms are currently supported
             querypt1 += "{\n?s ?p ?o . \n} UNION {\n" + querypt_class_rels + "\n}"
@@ -967,7 +1075,7 @@ class OntoEditor:
         query_body = "\n".join([querypt1, querypt_rels, querypt_nodes, query_rel_lim, querypt_ignore, querypt2])
         return query_body
 
-    def _render_by_label(self, graph: nx.MultiDiGraph, lang: str=None) -> nx.MultiDiGraph:
+    def _render_by_label(self, graph: nx.MultiDiGraph, lang: str = None) -> nx.MultiDiGraph:
         """ relabel the networkx graph's nodes and edges using the labels specified
         in the ontology (if there are labels available); defaults to first label
 
@@ -987,7 +1095,7 @@ class OntoEditor:
                 e[1]["label"] = label
         return graph
 
-    def _name_to_label(self, name: str, lang: str=None) -> str:
+    def _name_to_label(self, name: str, lang: str = None) -> str:
         """ return (first) label for an entity in the language specified
 
         :param elem: name of the ontology's element
@@ -1009,24 +1117,25 @@ class OntoEditor:
             label = name
         return label
 
-    def visualize(self, classes: list=None, properties: list=None, focusnode: str=None,
-                  radius: int=None, bylabel: bool=False, lang: str=None, open_html: bool=False) -> None:
+    def visualize(self, classes: list = None, properties: list = None, focusnode: str = None,
+                  radius: int = None, bylabel: bool = False, lang: str = None, open_html: bool = False) -> None:
         """ visualize onto as a graph; generates html
 
         :param classes: list of classes to be included in plot
         :param properties: list of properties to be included in plot
+        :param focusnode: node around which a partial graph shall be displayed
         :param radius: maximum number of relations between a node and a node of
             one of the classes specified
         :param bylabel: render visualization by labels (if available)
         :param lang: language of the labels to be displayed
+        :param open_html: open html file generated
         :return: None
         """
         # graph coloring settings; note that literals default to grey
         classcolor = "#0065bd"
         instancecolor = "#98c6ea"
-        coloring = {}
-        coloring[classcolor] = [c.name for c in self.onto.classes()]
-        coloring[instancecolor] = [i.name for i in self.onto.individuals()]
+        coloring = {classcolor: [c.name for c in self.onto.classes()],
+                    instancecolor: [i.name for i in self.onto.individuals()]}
 
         if not classes and not properties and not focusnode and not radius:
             graphdata = self._ntriples_to_df()
